﻿#-------------------------------------------------------------------------------
# Name:        if2bin8
# Purpose: GNSSのIF信号を指定ビット数ずつ取り出して保存し直す
#
# Author:      morishita
#
# Created:     20/02/2014
# Copyright:   (c) morishita 2014
# Licence:     MIT
#-------------------------------------------------------------------------------
import sys
import os
import math

bit_width = 3


def expand_bin_file(fpath, bw):
    """ bw bitサンプリングのIFデータが隙間なく格納されているバイナリデータをbw bitずつ切り出して0詰めした8 bitデータに変換し保存する
    Argv:
        fpath: <str> 対象のIFデータファイル
        bw:    <int> サンプリングビット数（分解能）
    """
    if not os.path.exists(fpath):
        sys.exit(1)
    if not os.path.isfile(fpath):
        sys.exit(1)
    root, ext = os.path.splitext(fpath)
    fw_name = root + "_8bin" + ext
    with open(fpath, "rb") as fr, open(fw_name, "wb") as fw:
        bin_data = fr.read()
        size = len(bin_data)
        print("size: " + str(size))
        upper = int(8 * size / bw)
        reserve = 0
        data = 0
        index = 0
        for k in range(upper):
            if reserve < bw:
                data = data << 8
                data += bin_data[index]
                index += 1
                reserve += 8
            hoge = reserve - bw     # 出力しない部分のビット数を計算
            out = data >> hoge
            data &= (2 ** hoge) - 1 # 出力済みの部分にマスクをかけて無視
            #print(out)
            fw.write(out.to_bytes(1, byteorder='big'))
            reserve -= bw           # 未出力のビット数を更新
            #input("")
    return


def main():
    argvs = sys.argv                    # コマンドライン引数を格納したリストの取得
    if len(argvs) > 1:
        fpath = argvs[1]                # 0番目はスクリプト名
    else:
        fpath = "bin_sample.bin"        # サンプルデータを作る
        with open(fpath, "wb") as fw:
            import random
            for i in range(500):
                ran = random.getrandbits(8)
                fw.write(ran.to_bytes(1, byteorder='big'))
    expand_bin_file(fpath, bit_width)



if __name__ == '__main__':
    main()
